# -*- coding: utf-8 -*-
r"""
This constants library holds the test signatures. Test signatures have different use cases:

- creating unique _temp_-folders for tests.

- identify test with single integer
"""
# Test Ids Multilayer Test =============================================================================================
MULTI_PRECISIONWITHOUTDAMPING = 1
MULTI_PRECISIONWITHDAMPING = 2
MULTI_DOMAINWALLS_Z = 3